#include <libsegy.h>
#include "segy_matlab.h"
#include <mex.h>
#include <matrix.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>


// mexfunction(out_argc, out_argv, in_argc, in_argv)
//   out_argc: int = number of output arguments requested by caller
//   out_argv: mxArray * = hooks to assign output args to
//   in_argc : int = number of input arguments assigned by caller
//   in_argv : mxArray * = input argument pointers
//
//   in_argv[0] : int = error number
//   out_argv[0]: string = error message
//
void mexFunction(int out_argc,  mxArray *out_argv[], 
                 int in_argc, const mxArray *in_argv[])
{
  int err; 
  char *errmsg = NULL;

  // check input arguments
  if (in_argc < 1 || ! mxIsNumeric(in_argv[0]) || mxIsEmpty(in_argv[0])) {
    mexPrintf("segy_error_message: input must be a number\n");
    return;
  }

  // check ouput arguments
  if (out_argc < 1) {
    mexPrintf("segy_error_message: must have one output argument\n");  
    return;
  }

  err = (int)mxGetScalar(in_argv[0]);
  errmsg = segy_error_message(err);
  if (! errmsg) {
    mexPrintf("segy_error_message: unknown error number");
    return;
  }

  if (! (out_argv[0] = mxCreateString(errmsg)) ) {
    mexPrintf("segy_error_message: out of memory");
    return;
  }
}
