#include <libsegy.h>
#include "segy_matlab.h"
#include <mex.h>
#include <matrix.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

// mexfunction(out_argc, out_argv, in_argc, in_argv)
//   out_argc: int = number of output arguments requested by caller
//   out_argv: mxArray * = hooks to assign output args to
//   in_argc : int = number of input arguments assigned by caller
//   in_argv : mxArray * = input argument pointers
//
//   out_argv[0]: struct = MATLAB struct with properly formatted fields
//
void mexFunction(int out_argc,  mxArray *out_argv[], 
                 int in_argc, const mxArray *in_argv[])
{
  mwSize size[2] = {0,0};

  // check ouput arguments
  if (out_argc < 1) {
    mexPrintf("segy_new_file_header: must have one output argument\n");
    return;
  }

  // create struct with fieldnames
  out_argv[0] = mxCreateStructMatrix(1, 1, 87, segy_trace_header_names);
  if (! out_argv[0]) {
    mexPrintf("segy_new_file_header: out of memory\n");
    return;
  }

  // fill in default values of the proper type
  mxSetField(out_argv[0], 0, "LineSeqNumber", mxCreateNumericMatrix(1,1,mxINT32_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "ReelSeqNumber", mxCreateNumericMatrix(1,1,mxINT32_CLASS,mxREAL));
  
  mxSetField(out_argv[0], 0, "FieldRecordNumber", mxCreateNumericMatrix(1,1,mxINT32_CLASS,mxREAL));
  
  mxSetField(out_argv[0], 0, "TraceNumber", mxCreateNumericMatrix(1,1,mxINT32_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "EnergySourceNumber", mxCreateNumericMatrix(1,1,mxINT32_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "CDPEnsembleNumber", mxCreateNumericMatrix(1,1,mxINT32_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "CDPTraceNumber", mxCreateNumericMatrix(1,1,mxINT32_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "TraceCode", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "VertSumTraces", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "HorizSumTraces", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "DataUse", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "SourceGroupDist", mxCreateNumericMatrix(1,1,mxINT32_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "GroupElevation", mxCreateNumericMatrix(1,1,mxINT32_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "SourceElevation", mxCreateNumericMatrix(1,1,mxINT32_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "SourceDepth", mxCreateNumericMatrix(1,1,mxINT32_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "GroupDatumElevation", mxCreateNumericMatrix(1,1,mxINT32_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "SourceDatumElevation", mxCreateNumericMatrix(1,1,mxINT32_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "SourceWaterDepth", mxCreateNumericMatrix(1,1,mxINT32_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "GroupWaterDepth", mxCreateNumericMatrix(1,1,mxINT32_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "ElevationScalar", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "CoordinateScalar", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "SourceX", mxCreateNumericMatrix(1,1,mxINT32_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "SourceY", mxCreateNumericMatrix(1,1,mxINT32_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "GroupX", mxCreateNumericMatrix(1,1,mxINT32_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "GroupY", mxCreateNumericMatrix(1,1,mxINT32_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "CoordinateUnit", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "WeatheringVelocity", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "SubweatheringVelocity", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "SourceTime", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "GroupTime", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "SourceCorrection", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "GroupCorrection", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "TotalCorrection", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "LagTimeA", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "LagTimeB", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "DelayAccordingTime", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "BruteTime", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "MuteTime", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "SamplesPerTrace", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "SampleInterval", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "GainType", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "GainConstant", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "InitialGain", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "Correlated", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "StartSweepFreq", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "EndSweepFreq", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "SweepLength", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "SweepType", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "StartSweepTaperLength", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "EndSweepTaperLength", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "TaperType", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "AliasFilterFreq", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "AliasFilterSlope", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "NotchFilterFreq", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "NotchFilterSlope", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "LowCutFreq", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "HightCutFreq", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "LowCutSlope", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "HighCutSlope", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "Year", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "DayOfYear", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "Hour", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "Minute", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "Second", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "TimeCode", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "TraceWeightFactor", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "GroupRollNumber", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "GroupTraceStart", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "GroupTraceEnd", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "GapSize", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "Overtravel", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));


  mxSetField(out_argv[0], 0, "CDPEnsembleX", mxCreateNumericMatrix(1,1,mxINT32_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "CDPEnsembleY", mxCreateNumericMatrix(1,1,mxINT32_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "InLine", mxCreateNumericMatrix(1,1,mxINT32_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "CrossLine", mxCreateNumericMatrix(1,1,mxINT32_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "Shotpoint", mxCreateNumericMatrix(1,1,mxINT32_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "ShotpointScalar", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "MeasurementUnit", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  size[0] = 1; size[1] = 6;
  mxSetField(out_argv[0], 0, "TransductionConstant", mxCreateCharArray(2, size));

  mxSetField(out_argv[0], 0, "TransductionUnit", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "DeviceID", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "TimeScalar", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "SourceType", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  size[0] = 1; size[1] = 6;
  mxSetField(out_argv[0], 0, "SourceDirection", mxCreateCharArray(2, size));

  size[0] = 1; size[1] = 6;
  mxSetField(out_argv[0], 0, "SourceMeasurement", mxCreateCharArray(2, size));

  mxSetField(out_argv[0], 0, "SourceMeasurementUnit", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  size[0] = 1; size[1] = 8;
  mxSetField(out_argv[0], 0, "unassigned", mxCreateCharArray(2, size));
}
