#include <libsegy.h>
#include "segy_matlab.h"
#include <mex.h>
#include <matrix.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>


// mexfunction(out_argc, out_argv, in_argc, in_argv)
//   out_argc: int = number of output arguments requested by caller
//   out_argv: mxArray * = hooks to assign output args to
//   in_argc : int = number of input arguments assigned by caller
//   in_argv : mxArray * = input argument pointers
//
//   in_argv[0] : string = file name of .segy file
//   in_argv[1] : int    = open mode, one of SEGY_RDONLY, SEGY_RDWR
//   out_argv[0]: int    = open file descriptor of .segy file, or -1 on error
//
void mexFunction(int out_argc,  mxArray *out_argv[], 
                 int in_argc, const mxArray *in_argv[])
{
  int fd = 0; 
  char filename[256] = {0};

  // check input arguments
  if (in_argc < 2 || ! mxIsChar(in_argv[0]) || mxIsEmpty(in_argv[0]) ||
      ! mxIsNumeric(in_argv[1]) || mxIsEmpty(in_argv[1])) {
    mexPrintf("segy_open_file: input must be (filename, openmode)\n");
    return;
  }

  // check ouput arguments
  if (out_argc < 1) {
    mexPrintf("segy_open_file: must have one output argument\n");  
    return;
  }

  // open filename
  if (mxGetString(in_argv[0], filename, 255)) {
    mexPrintf("segy_read_file_header: error reading input string\n");    
    return;
  }
  
  fd = segy_open_file(filename, (int)mxGetScalar(in_argv[1]));
  
  // return file descriptor
  out_argv[0] = mxCreateNumericMatrix(1,1,mxINT32_CLASS, mxREAL);
  if (out_argv[0]) ((int*)mxGetPr(out_argv[0]))[0] = fd;
  else {
    mexPrintf("segy_open_file: out of memory\n");    
    return;
  }
}
