#include "libsegy.h"
#include "segy_matlab.h"
#include <mex.h>
#include <matrix.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>


// mexfunction(out_argc, out_argv, in_argc, in_argv)
//   out_argc: int = number of output arguments requested by caller
//   out_argv: mxArray * = hooks to assign output args to
//   in_argc : int = number of input arguments assigned by caller
//   in_argv : mxArray * = input argument pointers
//
//   in_argv[0] : int = open file descriptor
//   in_argv[1] : int = trace number to seek to the start of
//   out_argv[0]: int = error code, 0 on success, nonzero on error
//
void mexFunction(int out_argc,  mxArray *out_argv[], 
                 int in_argc, const mxArray *in_argv[])
{
  int err, fd, trace;

  // check input arguments
  if (in_argc < 2 || ! mxIsNumeric(in_argv[0]) || ! mxIsNumeric(in_argv[1]) 
      || mxIsEmpty(in_argv[0]) || mxIsEmpty(in_argv[1])) {
    mexPrintf("segy_seek_trace: input is 2 integers\n");
    return;
  }
  fd = (int)mxGetScalar(in_argv[0]);
  trace = (int)mxGetScalar(in_argv[1]);

  // check output arguments
  if (out_argc < 1) {
    mexPrintf("segy_seek_trace: must have one output argument\n");
    return;
  }
  
  // call function
  err = segy_seek_trace(fd, trace);

  // store error return
  if (! (out_argv[0] = mxCreateNumericMatrix(1,1,mxINT32_CLASS,mxREAL)) ) {
    mexPrintf("segy_seek_trace: out of memory\n");
    return;
  }
  ((int*)mxGetPr(out_argv[0]))[0] = err;

  if (err) mexPrintf(segy_error_message(err));
}
