#include <libsegy.h>
#include "segy_matlab.h"
#include <mex.h>
#include <matrix.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

// mexfunction(out_argc, out_argv, in_argc, in_argv)
//   out_argc: int = number of output arguments requested by caller
//   out_argv: mxArray * = hooks to assign output args to
//   in_argc : int = number of input arguments assigned by caller
//   in_argv : mxArray * = input argument pointers
//
//   in_argv[0] : int    = open file descriptor of .segy file
//   in_argv[1] : struct = see return value of segy_read_file_header
//   out_argv[0]: int    = 0 on success, 1 on failure
//
void mexFunction(int out_argc,  mxArray *out_argv[], 
                 int in_argc, const mxArray *in_argv[])
{
  int in_fd = 0; int err = 0; int i;
  segy_file_header header;
  mxArray *field;

  // check input arguments
  if (in_argc < 2 || ! mxIsInt32(in_argv[0]) || ! mxIsStruct(in_argv[1])) {
    mexPrintf("segy_write_file_header: input must be file descriptor and struct\n");
    return;
  }
  in_fd = ((int*)mxGetPr(in_argv[0]))[0];

  // check ouput arguments
  if (out_argc < 1) {
    mexPrintf("segy_read_file_header: must have one output argument\n");
    return;
  }

  // copy MATLAB struct values into C struct fields
  if ( (field = mxGetField(in_argv[1], 0, "TextHeader"))
      && ! mxIsEmpty(field) && mxIsChar(field)) {
    for (i=0; i<3200; i++) 
      header.TextHeader[i] = ((short*)mxGetPr(field))[i]; 
  }
  else {
    mexPrintf("Error: field TextHeader invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[1], 0, "JobID"))
      && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.JobID = (int)mxGetScalar(field);
  else {
    mexPrintf("Error: field JobID invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[1], 0, "LineNumber"))
      && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.LineNumber = (int)mxGetScalar(field);
  else {
    mexPrintf("Error: field LineNumber invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[1], 0, "ReelNumber"))
      && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.ReelNumber = (int)mxGetScalar(field);
  else {
    mexPrintf("Error: field ReelNumber invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[1], 0, "TracesPerRecord"))
      && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.TracesPerRecord = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field TracesPerRecord invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[1], 0, "AuxTracesPerRecord"))
      && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.AuxTracesPerRecord = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field AuxTracesPerRecord invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[1], 0, "ReelSampleInterval"))
      && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.ReelSampleInterval = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field ReelSampleInterval invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[1], 0, "FieldSampleInterval"))
      && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.FieldSampleInterval = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field FieldSampleInterval invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[1], 0, "ReelSamplesPerTrace"))
      && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.ReelSamplesPerTrace = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field ReelSamplesPerTrace invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[1], 0, "FieldSamplesPerTrace"))
      && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.FieldSamplesPerTrace = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field FieldSamplesPerTrace invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[1], 0, "SampleFormatCode"))
      && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.SampleFormatCode = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field SampleFormatCode invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[1], 0, "CDPFold"))
      && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.CDPFold = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field CDPFold invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[1], 0, "TraceSortingCode"))
      && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.TraceSortingCode = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field TraceSortingCode invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[1], 0, "VertSumCode"))
      && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.VertSumCode = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field VertSumCode invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[1], 0, "StartSweepFreq"))
      && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.StartSweepFreq = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field StartSweepFreq invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[1], 0, "EndSweepFreq"))
      && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.EndSweepFreq = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field EndSweepFreq invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[1], 0, "SweepLength"))
      && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.SweepLength = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field SweepLength invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[1], 0, "SweepCode"))
      && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.SweepCode = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field SweepCode invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[1], 0, "SweepTraceNumber"))
      && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.SweepTraceNumber = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field SweepTraceNumber invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[1], 0, "StartSweepTaperLength"))
      && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.StartSweepTaperLength = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field StartSweepTaperLength invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[1], 0, "EndSweepTaperLength"))
      && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.EndSweepTaperLength = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field EndSweepTaperLength invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[1], 0, "TaperType"))
      && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.TaperType = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field TaperType invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[1], 0, "DataTracesCorrelated"))
      && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.DataTracesCorrelated = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field DataTracesCorrelated invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[1], 0, "BinaryGainRecovered"))
      && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.BinaryGainRecovered = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field BinaryGainRecovered invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[1], 0, "AmpRecoverMethod"))
      && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.AmpRecoverMethod = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field AmpRecoverMethod invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[1], 0, "MeasurementSystem"))
      && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.MeasurementSystem = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field MeasurementSystem invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[1], 0, "ImpulseSignalPolarity"))
      && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.ImpulseSignalPolarity = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field ImpulseSignalPolarity invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[1], 0, "VibratoryPolarityCode"))
      && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.VibratoryPolarityCode = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field VibratoryPolarityCode invalid");
    return;
  }


  if ( (field = mxGetField(in_argv[1], 0, "unassigned"))
      && ! mxIsEmpty(field) && mxIsChar(field)) {
    for (i=0; i<240; i++) 
      header.unassigned[i] = ((short*)mxGetPr(field))[i]; 
  }
  else {
    mexPrintf("Error: field unassigned invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[1], 0, "RevisionNumber"))
      && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.RevisionNumber = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field RevisionNumber invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[1], 0, "FixedLengthTraceFlag"))
      && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.FixedLengthTraceFlag = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field FixedLengthTraceFlag invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[1], 0, "ExtendedTextHeaders"))
      && ! mxIsEmpty(field) && mxIsNumeric(field))
    header.ExtendedTextHeaders = (short)mxGetScalar(field);
  else {
    mexPrintf("Error: field ExtendedTextHeaders invalid");
    return;
  }

  if ( (field = mxGetField(in_argv[1], 0, "unassigned2"))
      && ! mxIsEmpty(field) && mxIsChar(field)) {
    for (i=0; i<94; i++) 
      header.unassigned2[i] = ((short*)mxGetPr(field))[i]; 
  }
  else {
    mexPrintf("Error: field unassigned2 invalid");
    return;
  }


  // call C function and return result
  err = segy_write_file_header(in_fd, &header);
  out_argv[0] = mxCreateNumericMatrix(1,1,mxINT32_CLASS,mxREAL);
  if (out_argv[0]) ((int*)mxGetPr(out_argv[0]))[0] = err;
}
