#include <libsegy.h>
#include <stdlib.h>
#include <stdio.h>

static char hex[9] = {0};

const char * printbyte(char *byte) 
{
  char lo = (*byte) & 0x0f;
  char hi = ((*byte) >> 4) & 0x0f;
  if (lo < 10) lo += 48;
  else lo += 87;
  if (hi < 10) hi += 48;
  else hi += 87;
  hex[0] = hi;
  hex[1] = lo;
  hex[2] = 0;
  return hex;
}

const char * printshort(short *s) 
{
  char *sc = (char*)s;
  char result[5] = {0};
  snprintf(&result[0], 3, "%s", printbyte(&sc[0]));
  snprintf(&result[2], 3, "%s", printbyte(&sc[1]));
  snprintf(hex, 5, "%s", result);
  return hex;
}

const char * printint(int *i) 
{
  char *ci = (char*)i;
  char result[9] = {0};
  snprintf(&result[0], 3, "%s", printbyte(&ci[0]));
  snprintf(&result[2], 3, "%s", printbyte(&ci[1]));
  snprintf(&result[4], 3, "%s", printbyte(&ci[2]));
  snprintf(&result[6], 3, "%s", printbyte(&ci[3]));
  snprintf(hex, 9, "%s", result);
  return hex;
}

const char * printfloat2(float *f) 
{
  char *cf = (char*)f;
  char result[9] = {0};
  snprintf(&result[0], 3, "%s", printbyte(&cf[0]));
  snprintf(&result[2], 3, "%s", printbyte(&cf[1]));
  snprintf(&result[4], 3, "%s", printbyte(&cf[2]));
  snprintf(&result[6], 3, "%s", printbyte(&cf[3]));
  snprintf(hex, 9, "%s", result);
  return hex;
}


// printbits(byte, buf, len)
//   byte: char   = a byte of data
//   buf : char * = string buffer
//   len : size_t = length of buf in characters
// prints string representation of byte to buf, up to len-1 characters 
void printbits(char byte, char *buf, size_t len)
{
  int val[8] = {0};
  val[0] = (byte & 0x01) ? 1 : 0;
  val[1] = (byte & 0x02) ? 1 : 0;
  val[2] = (byte & 0x04) ? 1 : 0;
  val[3] = (byte & 0x08) ? 1 : 0;
  val[4] = (byte & 0x10) ? 1 : 0;
  val[5] = (byte & 0x20) ? 1 : 0;
  val[6] = (byte & 0x40) ? 1 : 0;
  val[7] = (byte & 0x80) ? 1 : 0;
  snprintf(buf, len, "%d%d%d%d%d%d%d%d", val[7], val[6], val[5], val[4],
	   val[3], val[2], val[1], val[0]);
}


// printfloat(f, buf, len)
//   f  : float * = data value
//   buf: char *  = bit string buffer
//   len: int = length of buf
// prints bits of float in native machine order to buf string
void printfloat(float *f, char *buf, size_t len) 
{
  char *byte = (char*)f;
  char bits[33] = {0};
  printbits(byte[0], bits, 9);
  printbits(byte[1], bits+8, 9);
  printbits(byte[2], bits+16, 9);
  printbits(byte[3], bits+24, 9);

  snprintf(buf, len, "%32s", bits); 
}


// printieee(f, buf, len)
//   f  : float * = data value (little-endian)
//   buf: char *  = string buffer
//   len: int     = length of buf
// prints description of ieee bits into string buf
void printieee(float *f, char *buf, int len)
{
  char bits[33] = {0};
  char rev[33] = {0};
  char des[80] = {0};
  printfloat(f, bits, 33);
  int i, byte;
  for (byte = 3; byte >= 0; byte--) {
    for (i = 0; i < 8; i++) rev[(8*(3 - byte)) + i] = bits[(8*byte) + i];
  }

  snprintf(des, 80, "s%cs e%c%c%c%c%c%c%c%ce m%sm", rev[0], rev[1], rev[2],
	   rev[3], rev[4], rev[5], rev[6], rev[7], rev[8], &rev[9]);
  snprintf(buf, len, "%s", des);
}


// printsegy(f, buf, len)
//   f  : float * = data value (big-endian)
//   buf: char *  = string buffer
//   len: int     = length of buf
// prints description of segy bits into string buf
void printsegy(float *f, char *buf, int len)
{
  char bits[33] = {0};
  char des[80] = {0};
  printfloat(f, bits, 33);

  snprintf(des, 80, "s%cs e%c%c%c%c%c%c%ce m%sm", bits[0], bits[1], bits[2],
	   bits[3], bits[4], bits[5], bits[6], bits[7], &bits[8]);
  snprintf(buf, len, "%s", des);
}


int main(int argc, char **argv) 
{
  char *USAGE = "USAGE: convert_float x.xxxexx\n";
  if (argc < 2) {
    printf("%s\n", USAGE);
    return 1;
  }

  char buf[80] = {0};
  float f = strtod(argv[1], NULL);
  float segy_f;
  float ieee_f;

  // do circular conversion
  convert2segy(1, (char*)&f, 4, &segy_f, 1);
  convert2ieee(1, (char*)&segy_f, 4, &ieee_f, 1);

  // print results
  printfloat(&f, buf, 33);
  printf("original(% 1.6e)  raw(%s)\n", f, buf);

  printf("original(% 1.6e)  hex(%s)\n", f, printfloat2(&f));

  printieee(&f, buf, 41);
  printf("original(% 1.6e) bits(%s)\n", f, buf);

  printsegy(&segy_f, buf, 41);
  printf("                   segy bits(%s)\n", buf);

  printieee(&ieee_f, buf, 41);
  printf("     new(% 1.6e) bits(%s)\n", ieee_f, buf);

  printfloat(&ieee_f, buf, 33);
  printf("     new(% 1.6e)  raw(%s)\n\n", ieee_f, buf);
  
  return 0;
}
