#include <stdio.h>      // printf
#include <sys/types.h>  // open
#include <sys/stat.h>   // open
#include <fcntl.h>      // open
#include <unistd.h>     // close
#include <math.h>       // fabs
#include <libsegy.h>

#ifndef O_BINARY
#define O_BINARY 0
#endif

int main(int argc, char **argv)
{
  int err = 0; int i;
  char *USAGE="USAGE: diffsegy segy_file1 segy_file2\n";
  if (argc < 3) {
    printf(USAGE);
    return 1;
  }

  // open files
  int fd1 = open(argv[1], O_RDONLY | O_BINARY);
  int fd2 = open(argv[2], O_RDONLY | O_BINARY);
  if (fd1 < 0 || fd2 < 0) {
    if (fd1 < 0) printf("libsegy: Error opening file %s\n", argv[1]);
    if (fd2 < 0) printf("libsegy: Error opening file %s\n", argv[2]);
    return 1;
  }

  // read file headers
  segy_file_header fhead1, fhead2;
  if ( (err = segy_read_file_header(fd1, &fhead1)) || 
       (err = segy_read_file_header(fd2, &fhead2)) ) {
    printf("%s\n", segy_error_message(err));
    return 1;
  }

  // compare file headers
  int fhead_equal = 1;
  /*
  for (i = 0; i < (int)sizeof(segy_file_header); i++) {
    char *bytes1 = (char*)&fhead1;
    char *bytes2 = (char*)&fhead2;
    if (bytes1[i] != bytes2[i]) {
      fhead_equal = 0;
      break;
    }
  }
  */
  if (! fhead_equal) {
    printf("File headers not equal:\n");
    for (i = 0; i < (int)sizeof(segy_file_header); i++) {
      char *bytes1 = (char*)&fhead1;
      char *bytes2 = (char*)&fhead2;
      if (bytes1[i] != bytes2[i])
	printf("  byte %5d: file1(%hhx) file2(%hhx)\n",i, bytes1[i], bytes2[i]);
    }
    return 1;
  }

  // compare trace headers/data
  segy_trace_header thead1, thead2;
  float data1[1024] = {0};
  float data2[1024] = {0};
  int trace_num = 0;
  while (err == 0) {
    err = segy_read_trace(fd1, 1, &thead1, data1, 1024);
    if (err) break;
    err = segy_read_trace(fd2, 1, &thead2, data2, 1024);
    if (err) break;

    // compare trace headers
    int thead_equal = 1;
    /*
    for (i = 0; i < (int)sizeof(segy_trace_header); i++) {
      char *bytes1 = (char*)&thead1;
      char *bytes2 = (char*)&thead2;
      if (bytes1[i] != bytes2[i]) {
	thead_equal = 0;
	break;
      }
    }
    */
    if (! thead_equal) {
      printf("Trace %5d headers not equal:\n", trace_num);
      for (i = 0; i < (int)sizeof(thead1); i++) {
	char *bytes1 = (char*)&thead1;
	char *bytes2 = (char*)&thead2;
	if (bytes1[i] != bytes2[i])
	  printf("  byte %5d: file1(%hhx) file2(%hhx)\n",i, bytes1[i], bytes2[i]);
      }
      return 1;
    }

    // compare trace data
    int data_equal = 1;
    for (i = 0; i < 1024; i++) {
      float diff = fabs(data1[i] - data2[i]);
      float smaller = data1[i];
      if (data2[i] < smaller) smaller = data2[i];
      float err = fabs(0.001 * smaller);
      if (diff > err) data_equal = 0;
      break;
    }
    if (! data_equal) {
      printf("Trace %5d data not equal:\n", trace_num);
      for (i = 0; i < 1024; i++) {
        if (data1[i] != data2[i])
	  printf("  datum %5d: file1(% 1.6e) file2(% 1.6e)\n", i, 
		 data1[i], data2[i]);
      }
      return 1;
    }

    /**
    for (i = 0; i < 4096; i++) {
      char *buf1 = (char*)data1;
      char *buf2 = (char*)data2;
      if (buf1[i] != buf2[i]) {
	printf("Trace bytes %d not equal: file1(%hhx) file2(%hhx)\n",
	       i, buf1[i], buf2[i]);
	return 1;
      }
    }
    **/
    trace_num++;
  }

  close(fd1);
  close(fd2);

  if (err == SEGY_EOF) {
    printf("files %s and %s are equal\n", argv[1], argv[2]);
    return 0;
  }

  printf("%s\n", segy_error_message(err));
  return 1;
}
